const char uuid_lua[] =
"-- uuid.lua (internal file)\n"
"\n"
"local ffi = require(\"ffi\")\n"
"local static_alloc = require('buffer').static_alloc\n"
"local builtin = ffi.C\n"
"\n"
"ffi.cdef[[\n"
"void\n"
"tt_uuid_create(struct tt_uuid *uu);\n"
"int\n"
"tt_uuid_from_string(const char *in, struct tt_uuid *uu);\n"
"void\n"
"tt_uuid_to_string(const struct tt_uuid *uu, char *out);\n"
"void\n"
"tt_uuid_bswap(struct tt_uuid *uu);\n"
"bool\n"
"tt_uuid_is_nil(const struct tt_uuid *uu);\n"
"bool\n"
"tt_uuid_is_equal(const struct tt_uuid *lhs, const struct tt_uuid *rhs);\n"
"char *\n"
"tt_uuid_str(const struct tt_uuid *uu);\n"
"int\n"
"tt_uuid_compare(const struct tt_uuid *a, const struct tt_uuid *b);\n"
"extern const struct tt_uuid uuid_nil;\n"
"]]\n"
"\n"
"local uuid_t = ffi.typeof('struct tt_uuid')\n"
"local UUID_STR_LEN = 36\n"
"local UUID_LEN = ffi.sizeof(uuid_t)\n"
"\n"
"local is_uuid = function(value)\n"
"    return ffi.istype(uuid_t, value)\n"
"end\n"
"\n"
"local uuid_tostring = function(uu)\n"
"    if not is_uuid(uu) then\n"
"        return error('Usage: uuid:str()')\n"
"    end\n"
"    return ffi.string(builtin.tt_uuid_str(uu), UUID_STR_LEN)\n"
"end\n"
"\n"
"local uuid_fromstr = function(str)\n"
"    if type(str) ~= 'string' then\n"
"        error(\"fromstr(str)\")\n"
"    end\n"
"    local uu = ffi.new(uuid_t)\n"
"    local rc = builtin.tt_uuid_from_string(str, uu)\n"
"    if rc ~= 0 then\n"
"        return nil\n"
"    end\n"
"    return uu\n"
"end\n"
"\n"
"local need_bswap = function(order)\n"
"    if order == nil or order == 'l' or order == 'h' or order == 'host' then\n"
"        return false\n"
"    elseif order == 'b' or order == 'n' or order == 'network' then\n"
"        return true\n"
"    else\n"
"        error('invalid byteorder, valid is l, b, h, n')\n"
"    end\n"
"end\n"
"\n"
"local uuid_tobin = function(uu, byteorder)\n"
"    if not is_uuid(uu) then\n"
"        return error('Usage: uuid:bin([byteorder])')\n"
"    end\n"
"    if need_bswap(byteorder) then\n"
"        local uuidbuf = static_alloc('struct tt_uuid')\n"
"        ffi.copy(uuidbuf, uu, UUID_LEN)\n"
"        builtin.tt_uuid_bswap(uuidbuf)\n"
"        return ffi.string(ffi.cast('char *', uuidbuf), UUID_LEN)\n"
"    end\n"
"    return ffi.string(ffi.cast('char *', uu), UUID_LEN)\n"
"end\n"
"\n"
"local uuid_frombin = function(bin, byteorder)\n"
"    if type(bin) ~= 'string' or #bin ~= UUID_LEN then\n"
"        error(\"frombin(bin, [byteorder])\")\n"
"    end\n"
"    local uu = ffi.new(uuid_t)\n"
"    ffi.copy(uu, bin, UUID_LEN)\n"
"    if need_bswap(byteorder) then\n"
"        builtin.tt_uuid_bswap(uu)\n"
"    end\n"
"    return uu\n"
"end\n"
"\n"
"local uuid_isnil = function(uu)\n"
"    if not is_uuid(uu) then\n"
"        return error('Usage: uuid:isnil()')\n"
"    end\n"
"    return builtin.tt_uuid_is_nil(uu)\n"
"end\n"
"\n"
"local fromstr = function(str)\n"
"    local uu = static_alloc('struct tt_uuid')\n"
"    local rc = builtin.tt_uuid_from_string(str, uu)\n"
"    if rc ~= 0 then\n"
"        return nil\n"
"    end\n"
"    return uu\n"
"end\n"
"\n"
"local to_uuid = function(value)\n"
"    if is_uuid(value) then\n"
"        return value\n"
"    end\n"
"    if type(value) == 'string' then\n"
"        return fromstr(value)\n"
"    end\n"
"    return nil\n"
"end\n"
"\n"
"local check_uuid = function(value, index, err_lvl)\n"
"    value = to_uuid(value)\n"
"    if value ~= nil then\n"
"        return value\n"
"    end\n"
"\n"
"    local err_fmt = 'incorrect value to convert to uuid as %d argument'\n"
"    error(err_fmt:format(index), err_lvl)\n"
"end\n"
"\n"
"local uuid_eq = function(lhs, rhs)\n"
"    rhs = to_uuid(rhs)\n"
"    if rhs == nil then\n"
"        return false\n"
"    end\n"
"    lhs = check_uuid(lhs, 1, 3)\n"
"    return builtin.tt_uuid_is_equal(lhs, rhs)\n"
"end\n"
"\n"
"local uuid_new = function()\n"
"    local uu = ffi.new(uuid_t)\n"
"    builtin.tt_uuid_create(uu)\n"
"    return uu\n"
"end\n"
"\n"
"local uuid_new_bin = function(byteorder)\n"
"    local uuidbuf = static_alloc('struct tt_uuid')\n"
"    builtin.tt_uuid_create(uuidbuf)\n"
"    return uuid_tobin(uuidbuf, byteorder)\n"
"end\n"
"local uuid_new_str = function()\n"
"    local uuidbuf = static_alloc('struct tt_uuid')\n"
"    builtin.tt_uuid_create(uuidbuf)\n"
"    return uuid_tostring(uuidbuf)\n"
"end\n"
"\n"
"local uuid_cmp = function(lhs, rhs)\n"
"    lhs = check_uuid(lhs, 1, 4)\n"
"    rhs = check_uuid(rhs, 2, 4)\n"
"    return builtin.tt_uuid_compare(lhs, rhs)\n"
"end\n"
"local uuid_lt = function(lhs, rhs)\n"
"    return uuid_cmp(lhs, rhs) < 0\n"
"end\n"
"local uuid_le = function(lhs, rhs)\n"
"    return uuid_cmp(lhs, rhs) <= 0\n"
"end\n"
"\n"
"local uuid_mt = {\n"
"    __tostring = uuid_tostring;\n"
"    __eq = uuid_eq;\n"
"    __lt = uuid_lt;\n"
"    __le = uuid_le;\n"
"    __index = {\n"
"        isnil = uuid_isnil;\n"
"        bin   = uuid_tobin;    -- binary host byteorder\n"
"        str   = uuid_tostring; -- RFC4122 string\n"
"    }\n"
"}\n"
"\n"
"ffi.metatype(uuid_t, uuid_mt)\n"
"\n"
"return setmetatable({\n"
"    NULL        = builtin.uuid_nil;\n"
"    new         = uuid_new;\n"
"    fromstr     = uuid_fromstr;\n"
"    frombin     = uuid_frombin;\n"
"    bin         = uuid_new_bin;   -- optimized shortcut for new():bin()\n"
"    str         = uuid_new_str;   -- optimized shortcut for new():str()\n"
"    is_uuid     = is_uuid;\n"
"}, {\n"
"    __call = uuid_new; -- shortcut for new()\n"
"})\n"
""
;
